xChar = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")

local L = AceLibrary("AceLocale-2.2"):new("xChar")
local T = AceLibrary("Gratuity-2.0")
local BB = AceLibrary("Babble-Boss-2.2")
local BZ = AceLibrary("Babble-Zone-2.2")

UploadData = {}

-- http://wowace.com/wiki/AceOptions_data_table
xChar.options = {
	type = "group",
	args = {
		getAllData = {
			type = "execute",
			name = "getAllData",
			desc = L["collects all character information now."],
			func = function() xChar:getAllData() end,
		}
	}
}

function xChar:OnEnable()
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	self:RegisterEvent("PLAYER_LEVEL_UP")
	self:RegisterEvent("BANKFRAME_CLOSED")
	self:RegisterEvent("UNIT_QUEST_LOG_CHANGED")
	self:RegisterEvent("HONOR_CURRENCY_UPDATE")
	self:RegisterEvent("PLAYER_PVP_RANK_CHANGED")
	self:RegisterEvent("CRAFT_SHOW")
	self:RegisterEvent("CHARACTER_POINTS_CHANGED")
	self:RegisterEvent("PLAYER_GUILD_UPDATE")
	self:RegisterEvent("SHOW_COMPARE_TOOLTIP")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	self:RegisterEvent("FRIENDLIST_UPDATE")
	self:RegisterEvent("ARENA_TEAM_ROSTER_UPDATE")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
	UploadData = {}
	
	self.currentBoss = nil
	
	local version = GetAddOnMetadata("XCHAR", "Version") or "test"
	self:Print("XCHAR version ".. version .."  Loaded|r")
	
	UploadData.HeartLocation = GetBindLocation()
	UploadData.XcharVer = "0.93"
	self:createBossTable()
	
	self:RegisterChatCommand({"/xc", "/xchar"}, self.options)
	
	self:ScheduleEvent(function() self:getAllData() end, 10, self)

end

function xChar:getAllData()
	self:GetPVP()
	self:GetArena()
	self:GetProfileData()
	
	self:GetBaseStats()
	self:GetMeeleStats()
	self:GetSpellStats()
	self:GetDefenceStat()
	self:GetRangedStat()
	
	self:GetAddons()
	self:GetBags()
	self:GetInventoryItems()
	self:GetTalents()
	self:GetTalentShort()
	self:GetSkill()
	self:GetReputation()
	self:GetGuildInfo()
	self:GetBuffs()
	self:GetQuests()
	self:GetFriends()
--	GetGameTime()
--	GetTime()
	RequestTimePlayed()
	
	
end


function xChar:UNIT_INVENTORY_CHANGED()
	self:GetInventoryItems()
	self:GetBags()
	
end

function xChar:PLAYER_LEVEL_UP()
	UploadData.Level = UnitLevel("player")

end

function xChar:BANKFRAME_CLOSED()
	self:GetBankItem()
	
end

function xChar:UNIT_QUEST_LOG_CHANGED(unit)
	if unit == "player" then
		self:GetQuests()
		
	end
end

function xChar:TIME_PLAYED_MSG()
	UploadData.TimePlayed = arg1
	UploadData.TimeAtLevel = arg2
	
end

function xChar:HONOR_CURRENCY_UPDATE()
	self:GetPVP()
	
end

function xChar:PLAYER_PVP_RANK_CHANGED()
	self:GetPVP()
	
end

function xChar:CRAFT_SHOW()
	self:GetSkill()
	
end

function xChar:CHARACTER_POINTS_CHANGED()
	self:GetProfileData()
	
end

function xChar:PLAYER_GUILD_UPDATE()
	self:GetGuildInfo()
	
end

function xChar:SHOW_COMPARE_TOOLTIP()
	self:Print("ITEM ".. arg1.. " : " .. arg2 .."  tooltip: " .. getglobal("ShoppingTooltip"..arg2));
end

function xChar:UNIT_INVENTORY_CHANGED()
	self:GetBaseStats()
	self:GetMeeleStats()
	self:GetSpellStats()
	self:GetDefenceStat()
	self:GetRangedStat()

	self:GetBags()
	self:GetInventoryItems()
	self:GetBuffs()
	
end

function xChar:FRIENDLIST_UPDATE()
	self:GetFriends()
	
end

function xChar:ARENA_TEAM_ROSTER_UPDATE()
	self:GetArenaTeamInfo(PVPTeamDetails.team)
	
end

function xChar:GetArenaTeamInfo(teamID)

	local ArenaTeam = {}

	local teamName, teamSize, teamRating, weekPlayed, weekWins, seasonPlayed, seasonWins, playerPlayed, teamRank, 
	bgR, bgG, bgB, emblem, emblemR, emblemG, emblemB, borderR, borderG, borderB = GetArenaTeam(teamID)
	
	ArenaTeam.teamName = teamName
	ArenaTeam.teamSize = teamSize
	ArenaTeam.teamRating = teamRating
	ArenaTeam.weekPlayed = weekPlayed
	ArenaTeam.weekWins = weekWins
	ArenaTeam.seasonPlayed = seasonPlayed
	ArenaTeam.seasonWins = seasonWins
	ArenaTeam.playerPlayed = playerPlayed
	ArenaTeam.bgR = bgR
	ArenaTeam.bgG = bgG
	ArenaTeam.bgB = bgB
	ArenaTeam.emblem = emblem
	ArenaTeam.emblemR = emblemR
	ArenaTeam.emblemG = emblemG
	ArenaTeam.emblemB = emblemB
	ArenaTeam.borderR = borderR
	ArenaTeam.borderG = borderG
	ArenaTeam.borderB = borderB

	local membercount = GetNumArenaTeamMembers(teamID, true)
	local Members = {}
	
	for playerID = 1, membercount do
	
		local name, rank, level, class, online, played, win, 
		seasonPlayed, seasonWin = GetArenaTeamRosterInfo(teamID, playerID)
		
		Members[playerID] = {}
		Members[playerID].name = name
		Members[playerID].rank = rank
		Members[playerID].level = level
		Members[playerID].class = class
		Members[playerID].online = online
		Members[playerID].weekPlayed = played
		Members[playerID].weekWin = win
		Members[playerID].seasonPlayed = seasonPlayed
		Members[playerID].seasonWin = seasonWin
	
	end
	
	ArenaTeam.Members = Members
	UploadData.ArenaTeam[teamID] = ArenaTeam

end

function xChar:GetArena()
	UploadData.pvp_arenapoints = GetArenaCurrency()
	UploadData.ArenaTeam = {}
	for i = 1, 3 do
		self:ScheduleEvent(
			function () UploadData.ArenaTeam[i] = {}; PVPTeamDetails.team = i; ArenaTeamRoster(i) end, 2 * i
		)	
	end	
end

function xChar:GetGuildInfo()
	if not IsInGuild() then return end
	GuildRoster()
	UploadData.guildmembers = {}
	
	for i = 1, GetNumGuildMembers(true) do
		local name, rank, rankIndex, level, class = GetGuildRosterInfo(i)
		UploadData.guildmembers[i] = {}
		UploadData.guildmembers[i].MName = name
--		UploadData.guildmembers[i].MRank = rank
		UploadData.guildmembers[i].MRankIndex = rankIndex
		UploadData.guildmembers[i].MLevel = level
		UploadData.guildmembers[i].MClass = class
	end	
end

function xChar:GetBankItem()
	UploadData.bank = {}

	local id = 1;
	local BagName = "BankSlot"

	UploadData.bank[id] = {}
	UploadData.bank[id].BagName = BagName
	UploadData.bank[id].BagSize = GetContainerNumSlots(-1)
	UploadData.bank[id].BagId = 1
	id = id + 1;
	
	for j = 1, GetContainerNumSlots(-1) do
		local ItemLink = GetContainerItemLink(-1, j)
		if ItemLink then
			local itemName, _, itemRarity, itemLevel, itemMinLevel, itemType, 
			itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(ItemLink)
			
			UploadData.bank[id] = {}
			UploadData.bank[id].BagItemName = itemTexture
			UploadData.bank[id].BagItemBagId = 1
			UploadData.bank[id].BagItemAmount = itemStackCount
			-- UploadData.bags[id].BagItemTooltip = self:GetBagTooltipInfo(-1, j)
			id = id + 1
		end
	end
	
	for i = 5, 11 do
		local BagName = GetBagName(i)
		if BagName then
			UploadData.bank[id] = {}
			UploadData.bank[id].BagName = BagName
			UploadData.bank[id].BagSize = GetContainerNumSlots(i)
			UploadData.bank[id].BagId = i
			id = id + 1;
			
			for j = 1, GetContainerNumSlots(i) do
				local ItemLink = GetContainerItemLink(i, j)
                
				if ItemLink then
					local itemName, _, itemRarity, itemLevel, itemMinLevel, itemType, 
					itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(ItemLink)

					local _, _, ItemId	= string.find(ItemLink,"^.*Hitem:(%d+)")
					UploadData.bank[id] = {}
					UploadData.bank[id].BagItemName = itemTexture
					UploadData.bank[id].BagItemBagId = i
					UploadData.bank[id].BagItemId = ItemId
					UploadData.bank[id].BagItemAmount = itemStackCount
					-- UploadData.bags[id].BagItemTooltip = self:GetBagTooltipInfo(i, j)
					id = id + 1;
				end
			end
		end
	end
end

function xChar:GetBuffs()
	local i = 1
	local first_in = 0
	UploadData.buffs = {}
	
	while UnitBuff("player", i) do
		if first_in == 0 then
			first_in = 1
			UploadData.buffs = {}
		end

		local name, rank, texture = UnitBuff("player", i)
		
		UploadData.buffs[i] = {}
		UploadData.buffs[i].BuffName = name
		UploadData.buffs[i].BuffRank = rank
		UploadData.buffs[i].BuffTexture = texture
		i = i + 1
	end
end

function xChar:GetFriends()
	if GetNumFriends() ~= 0 then
	UploadData.friends = {}
	
		for i = 1, GetNumFriends() do
			local name, level, class, area, connected, status = GetFriendInfo(i)
			UploadData.friends[i] = {}
			UploadData.friends[i].FriendName = name
			UploadData.friends[i].FriendLevel = level
			UploadData.friends[i].FriendClass = class
			UploadData.friends[i].FriendArea = area
			UploadData.friends[i].FriendConnected = connected
			UploadData.friends[i].FriendStatus = status
		end
	end
end
function xChar:GetBagTooltipInfo(BagId, Slot)
	T:SetBagItem(BagId, Slot)
	
	local TooltipText = ""
	for i = 1, T:NumLines() do
		local text = T:GetLine(i)
		if text then TooltipText = text .. " | " end 
	end
	return TooltipText
end

function xChar:GetBagTooltipInfoLegacy(BagId,Slot)
         ScanningTooltip:ClearLines()
         ScanningTooltip:SetBagItem(BagId,Slot);
         local TooltipText            = "";
         for i=1,ScanningTooltip:NumLines() do
             local text = getglobal("ScanningTooltipTextLeft" .. i):GetText();
             if string.find(text,".*>.*") == nil then
                TooltipText              = TooltipText ..text.."|";
             end
         end
         return TooltipText;
end
function xChar:GetSlotTooltipInfo(SlotName)
	T:SetInventoryItem("player", GetInventorySlotInfo(SlotName))
	local TooltipText = ""
	for i = 1, T:NumLines() do
		local text = T:GetLine(i)
		if text then TooltipText = text .. "|" end 
	end
	return TooltipText
end
function xChar:GetSlotTooltipInfoLegacy(SlotName)
         ScanningTooltip:ClearLines()
         ScanningTooltip:SetInventoryItem("player", GetInventorySlotInfo(SlotName));
         local TooltipText            = "";
         for i=1,ScanningTooltip:NumLines() do
             local text = getglobal("ScanningTooltipTextLeft" .. i):GetText();
             if string.find(text,".*>.*") == nil then
                TooltipText              = TooltipText ..text.."|";
             end
         end
         return TooltipText;
end
function xChar:GetTalentQuery()
	local TalentQuery = ""
	for i = 1, GetNumTalentTabs() do
		for j = 1, GetNumTalents(i) do
			local nameTalent, icon, tier, column, currRank, maxRank = GetTalentInfo(i, j)
			TalentQuery = TalentQuery .. currRank
		end
	end
	return TalentQuery
end

function xChar:GetQuests()
	if GetNumQuestLogEntries() ~= 0 then
		UploadData.quests = {}
		local j = 1
		for i = 1, GetNumQuestLogEntries() do
			local quest, level, questTag, group, isHeader, isCollapsed = GetQuestLogTitle(i)
			if quest and not isHeader then
				UploadData.quests[j] = {}
				UploadData.quests[j].quest_name = quest
				UploadData.quests[j].quest_level = level
				j = j + 1
			end
		end
	end
end

function xChar:GetSpellStats()
         UpdatePaperdollStats("PlayerStatFrameLeft", "PLAYERSTAT_SPELL_COMBAT");
         if getglobal("PlayerStatFrameLeft1") ~= nil then
            if getglobal("PlayerStatFrameLeft1").bonusDamage ~= nil then
            UploadData.spellBonusDamage_holy                = getglobal("PlayerStatFrameLeft1").bonusDamage[2];
            UploadData.spellBonusDamage_fire                = getglobal("PlayerStatFrameLeft1").bonusDamage[3];
            UploadData.spellBonusDamage_nature              = getglobal("PlayerStatFrameLeft1").bonusDamage[4];
            UploadData.spellBonusDamage_frost               = getglobal("PlayerStatFrameLeft1").bonusDamage[5];
            UploadData.spellBonusDamage_arcane              = getglobal("PlayerStatFrameLeft1").bonusDamage[7];
            UploadData.spellBonusDamage_shadow              = getglobal("PlayerStatFrameLeft1").bonusDamage[6];
            end
         end
         if getglobal("PlayerStatFrameLeft2").tooltip2 ~= nil then
            local _,_,sbh                                   = string.find(getglobal("PlayerStatFrameLeft2").tooltip2,"^.*to (%d+)");
            if sbh == nil then
               _,_,sbh                                      = string.find(getglobal("PlayerStatFrameLeft2").tooltip2,"^.*zu (%d+)");
            end
            UploadData.spellBonusHealing_value              = sbh;
         end
         if getglobal("PlayerStatFrameLeft3").tooltip ~= nil then
            local _,_,shr                                   = string.find(getglobal("PlayerStatFrameLeft3").tooltip,"^.* (%d+)");
            UploadData.spellHitRating_value                 = shr;
         end
         if  getglobal("PlayerStatFrameLeft3").tooltip2 ~= nil then
             local _,_,shrp                                  = string.find(getglobal("PlayerStatFrameLeft3").tooltip2,"^.*by (%d+\.%d+)");
             if shrp == nil then
                _,_,shrp                                  = string.find(getglobal("PlayerStatFrameLeft3").tooltip2,"^.*um (%d+\.%d+)");
             end
             UploadData.spellHitRating_increasedHitPercent   = shrp;
         end
         if getglobal("PlayerStatFrameLeft4") ~= nil then
            UploadData.spellCritChance_holy                 = getglobal("PlayerStatFrameLeft4").spellCrit[2];
            UploadData.spellCritChance_fire                 = getglobal("PlayerStatFrameLeft4").spellCrit[3];
            UploadData.spellCritChance_nature               = getglobal("PlayerStatFrameLeft4").spellCrit[4];
            UploadData.spellCritChance_frost                = getglobal("PlayerStatFrameLeft4").spellCrit[5];
            UploadData.spellCritChance_arcane               = getglobal("PlayerStatFrameLeft4").spellCrit[7];
            UploadData.spellCritChance_shadow               = getglobal("PlayerStatFrameLeft4").spellCrit[6];
         end
         if UnitHasMana("player") then
		local base, casting = GetManaRegen()
	        UploadData.spellManaRegen_notCasting            = floor(base * 5);
                UploadData.spellManaRegen_casting               = floor(casting * 5);
        end
         UploadData.spellPenetration_value                  = GetSpellPenetration();
         table.sort(UploadData);
end
function xChar:GetDefenceStat()
         UpdatePaperdollStats("PlayerStatFrameLeft", "PLAYERSTAT_DEFENSES");
         if getglobal("PlayerStatFrameLeft1").tooltip2 ~= nil then
            local _,_,daep                                   = string.find(getglobal("PlayerStatFrameLeft1").tooltip2,"^.* (%d+\.%d+)");
            UploadData.defensesArmor_percent                 = daep;
         end
         UploadData.defensesDefense_value                 = UnitDefense("player");
         if  getglobal("PlayerStatFrameLeft2").tooltip2 ~= nil then
             local _,_,ddr,ddpd,ddi,ddd                       = string.find(getglobal("PlayerStatFrameLeft2").tooltip2,"^.*Rating (%d+).*%+(%d).* by.(%d).* by.(%d)");
             if ddr == nil then
               _,_,ddr,ddpd,ddi,ddd                           = string.find(getglobal("PlayerStatFrameLeft2").tooltip2,"^.* (%d+).*%+(%d).* um.(%d).* um.(%d)");
             end
             UploadData.defensesDefense_rating                = ddr;
             UploadData.defensesDefense_plusDefense           = ddpd;
             UploadData.defensesDefense_increasePercent       = ddi;
             UploadData.defensesDefense_decreasePercent       = ddd;
         end
         if getglobal("PlayerStatFrameLeft3").tooltip ~= nil then
            local _,_,dddr                                   = string.find(getglobal("PlayerStatFrameLeft3").tooltip,"^.* (%d+)");
            UploadData.defensesDodge_rating                  = dddr;
         end
         if  getglobal("PlayerStatFrameLeft3").tooltip2 ~= nil then
             local _,_,dddrc                                  = string.find(getglobal("PlayerStatFrameLeft3").tooltip2,"^.* (%d+)");
             UploadData.defensesDodge_increasePercent         = dddrc;
             UploadData.defensesDodge_percent                 = GetDodgeChance();
             UploadData.defensesBlock_percent                 = GetBlockChance();
             UploadData.defensesParry_percent                 = GetParryChance();
         end
         if  getglobal("PlayerStatFrameLeft4").tooltip ~= nil then
             local _,_,ddpr                                   = string.find(getglobal("PlayerStatFrameLeft4").tooltip,"^.* (%d+)");
             UploadData.defensesParry_rating                  = ddpr;
         end
         if getglobal("PlayerStatFrameLeft4").tooltip2 ~= nil then
            local _,_,ddprc                                  = string.find(getglobal("PlayerStatFrameLeft4").tooltip2,"^.* (%d+\.%d+)");
            UploadData.defensesParry_increasePercent         = ddprc;
         end
         if getglobal("PlayerStatFrameLeft5").tooltip ~= nil then
            local _,_,ddbr                                   = string.find(getglobal("PlayerStatFrameLeft5").tooltip,"^.* (%d+)");
            UploadData.defensesBlock_rating                  = ddbr;
         end
         if getglobal("PlayerStatFrameLeft5").tooltip2 ~= nil then
            local _,_,ddbrc                                  = string.find(getglobal("PlayerStatFrameLeft5").tooltip2,"^.* (%d+\.%d+)");
            UploadData.defensesBlock_increasePercent         = ddbrc;
         end
         if getglobal("PlayerStatFrameLeft6").tooltip ~= nil then
            local _,_,ddrr                                   = string.find(getglobal("PlayerStatFrameLeft6").tooltip,"^.* (%d+)");
            UploadData.defensesResilience_value              = ddrr;
         end
         if  getglobal("PlayerStatFrameLeft6").tooltip2 then
             local _,_,ddbrh,ddbrd                            = string.find(getglobal("PlayerStatFrameLeft6").tooltip2,"^.*by (%d+\.%d+).*by (%d+\.%d+)");
             if ddbrh == nil then
                _,_,ddbrh,ddbrd                            = string.find(getglobal("PlayerStatFrameLeft6").tooltip2,"^.*um (%d+\.%d+).*um (%d+\.%d+)");
             end
             UploadData.defensesResilience_hitPercent         = ddbrh;
             UploadData.defensesResilience_damagePercent      = ddbrd;
         end
         table.sort(UploadData);
end
function xChar:GetRangedStat()
         UpdatePaperdollStats("PlayerStatFrameLeft", "PLAYERSTAT_RANGED_COMBAT");
         if getglobal("PlayerStatFrameLeft1").tooltip2 ~= nil then
            local _,_,rwr                                  = string.find(getglobal("PlayerStatFrameLeft1").tooltip2,"(%d+)");
            UploadData.rangedWeaponSkill_rating            = rwr;
         end
         if getglobal("PlayerStatFrameLeft1").tooltip ~= nil then
            local _,_,rwr                                  = string.find(getglobal("PlayerStatFrameLeft1").tooltip,"(%d+)");
            UploadData.rangedWeaponSkill_value             = rwr;
         end
         local meeleDamage2                             = getglobal("PlayerStatFrameLeft2");
         if meeleDamage2 ~= nil then
            if meeleDamage2.damage ~= nil then
               local _,_,damageMin,damageMax                  = string.find(meeleDamage2.damage,"^(%d+) .*(%d+)");
               local speed, lowDmg, hiDmg, posBuff, negBuff, percent = UnitRangedDamage("player")
               UploadData.rangedDamage_min                    = lowDmg;
               UploadData.rangedDamage_max                    = hiDmg;
               UploadData.rangedDamage_dps                    = meeleDamage2.dps;
               UploadData.rangedDamage_speed                  = meeleDamage2.attackSpeed;
            end
         end
         if getglobal("PlayerStatFrameLeft3") ~= nil then
            if getglobal("PlayerStatFrameLeft3").tooltip2 ~= nil then
               local _,_,meeleAttackrat,meeleAttackp          = string.find(getglobal("PlayerStatFrameLeft3").tooltip2,"^.*(%d+)\.*(%d+\.%d+)");
               UploadData.rangedSpeed_hasteRating          = meeleAttackrat;
               UploadData.rangedSpeed_hastePercent         = meeleAttackp;
            end
         end
         if getglobal("PlayerStatFrameLeft3").tooltip ~= nil then
            local _,_,meeleAttackspeed                     = string.find(getglobal("PlayerStatFrameLeft3").tooltip,"^.*(%d+\.%d+)");
            UploadData.rangedSpeed_value                   = meeleAttackspeed;
         end
         if getglobal("PlayerStatFrameLeft4").tooltip ~= nil then
            local _,_,mape                                 = string.find(getglobal("PlayerStatFrameLeft4").tooltip,"^.* (%d+)");
            UploadData.rangedPower_effective               = mape;
         end
         if getglobal("PlayerStatFrameLeft4").tooltip2 ~= nil then
            local _,_,mapp                                 = string.find(getglobal("PlayerStatFrameLeft4").tooltip2,"^.* (%d+\.%d+)");
            UploadData.rangedPower_increasedDps         = mapp;
         end
         if  getglobal("PlayerStatFrameLeft5").tooltip ~= nil then
             local _,_,mhe                                 = string.find(getglobal("PlayerStatFrameLeft5").tooltip,"^.* (%d+)");
             UploadData.rangedHitRating_value           = mhe;
         end
         if getglobal("PlayerStatFrameLeft5").tooltip2 ~= nil then
            local _,_,mhp                                          = string.find(getglobal("PlayerStatFrameLeft5").tooltip2,"^.* (%d+\.%d+)");
            UploadData.rangedHitRating_increasedHitPercent      = mhp;
         end
         if  getglobal("PlayerStatFrameLeft6").tooltip ~= nil then
             local _,_,mce                                          = string.find(getglobal("PlayerStatFrameLeft6").tooltip,"^.* (%d+\.%d+)");
             UploadData.rangedCritChance_percent                  = mce;
         end
         if getglobal("PlayerStatFrameLeft6").tooltip2  ~= nil then
            local _,_,mcr,mcp                                      = string.find(getglobal("PlayerStatFrameLeft6").tooltip2,"^.* (%d).*(%d+\.%d+)");
            UploadData.rangedCritChance_rating                   = mcr;
            UploadData.rangedCritChance_plusPercent              = mcp;
         end
end
function xChar:GetMeeleStats()
         local value,rating,_,_            = UnitAttackBothHands("player");
         UploadData.meleeMainHandWeaponSkill_value              = value;
         UploadData.meleeMainHandWeaponSkill_rating             = rating;
         UploadData.meleeMainHandSpeed_value                    = UnitAttackSpeed("player");
         UpdatePaperdollStats("PlayerStatFrameLeft", "PLAYERSTAT_MELEE_COMBAT");
         local meeleAttack                                      = getglobal("PlayerStatFrameLeft3");
         if meeleAttack.tooltip2 ~= nil then
            local _,_,meeleAttackrat,meeleAttackp                  = string.find(meeleAttack.tooltip2,"^.*(%d+)\.*(%d+\.%d+)");
            UploadData.meleeMainHandSpeed_hasteRating          = meeleAttackrat;
            UploadData.meleeMainHandSpeed_hastePercent         = meeleAttackp;
         end
         local meeleDamage2                                     = getglobal("PlayerStatFrameLeft2");
         if meeleDamage2 ~= nil then
         if meeleDamage2.damage ~= nil then
            local lowDmg, hiDmg, offlowDmg, offhiDmg, posBuff, negBuff, percentmod = UnitDamage("player")
               UploadData.meleeMainHandDamage_min                    = lowDmg;
               UploadData.meleeMainHandDamage_max                    = hiDmg;
               UploadData.meleeMainHandDamage_dps                     = meeleDamage2.dps;
               UploadData.meleeMainHandDamage_speed                   = meeleDamage2.attackSpeed;
         end
         end
         if getglobal("PlayerStatFrameLeft4").tooltip ~= nil then
            local _,_,mape                                         = string.find(getglobal("PlayerStatFrameLeft4").tooltip,"^.* (%d+)");
            UploadData.meleePower_effective                     = mape;
         end
         if getglobal("PlayerStatFrameLeft4").tooltip2 ~= nil then
            local _,_,mapp                                         = string.find(getglobal("PlayerStatFrameLeft4").tooltip2,"^.* (%d+\.%d+)");
            UploadData.meleePower_increasedDps                  = mapp;
         end
         if getglobal("PlayerStatFrameLeft5").tooltip ~= nil then
            local _,_,mhe                                          = string.find(getglobal("PlayerStatFrameLeft5").tooltip,"^.* (%d+)");
            UploadData.meleeHitRating_value                     = mhe;
         end
         if getglobal("PlayerStatFrameLeft5").tooltip2 ~= nil then
            local _,_,mhp                                          = string.find(getglobal("PlayerStatFrameLeft5").tooltip2,"^.* (%d+\.%d+)");
            UploadData.meleeHitRating_increasedHitPercent       = mhp;
         end
         if getglobal("PlayerStatFrameLeft6").tooltip ~= nil then
            local _,_,mce                                          = string.find(getglobal("PlayerStatFrameLeft6").tooltip,"^.* (%d+\.%d+)");
            UploadData.meleeCritChance_percent                  = mce;
         end
         if getglobal("PlayerStatFrameLeft6").tooltip2 ~= nil then
            local _,_,mcr,mcp                                      = string.find(getglobal("PlayerStatFrameLeft6").tooltip2,"^.* (%d).*(%d+\.%d+)");
            UploadData.meleeCritChance_rating                   = mcr;
            UploadData.meleeCritChance_plusPercent              = mcp;
         end
         table.sort(UploadData)
end
function xChar:GetBaseStats()
   UpdatePaperdollStats("PlayerStatFrameLeft", "PLAYERSTAT_BASE_STATS");
   local stat, effectiveStat, posBuff, negBuff = UnitStat("player", 1);
   UploadData.strength_effective                                = effectiveStat;
   UploadData.strength_base                                     = stat - posBuff - negBuff;
   stat, effectiveStat, posBuff, negBuff = UnitStat("player", 2);
   UploadData.agility_effective                                = effectiveStat;
   UploadData.agility_base                                     = stat - posBuff - negBuff;
   stat, effectiveStat, posBuff, negBuff = UnitStat("player", 3);
   UploadData.stamina_effective                                = effectiveStat;
   UploadData.stamina_base                                     = stat - posBuff - negBuff;
   stat, effectiveStat, posBuff, negBuff = UnitStat("player", 4);
   UploadData.intellect_effective                                = effectiveStat;
   UploadData.intellect_base                                     = stat - posBuff - negBuff;
   stat, effectiveStat, posBuff, negBuff = UnitStat("player", 5);
   UploadData.spirit_effective                                = effectiveStat;
   UploadData.spirit_base                                     = stat - posBuff - negBuff;
   local base, effectiveArmor, armor, posBuff, negBuff = UnitArmor("player");
   UploadData.armor_effective                                   = effectiveArmor;
   UploadData.armor_base                                        = base - posBuff - negBuff;
   UploadData.defensesArmor_effective                           = effectiveArmor;
   UploadData.defensesArmor_base                                = base - posBuff - negBuff;
   if  getglobal("PlayerStatFrameLeft1").tooltip2 ~= nil then
      local _,_,strat                                              = string.find(getglobal("PlayerStatFrameLeft1").tooltip2,"^.* by (%d+)");
      if strat == nil then
        _,_,strat                                              = string.find(getglobal("PlayerStatFrameLeft1").tooltip2,"^.* um (%d+)");
      end
      UploadData.strength_attack                                   = strat;
   end
   if  getglobal("PlayerStatFrameLeft2").tooltip2 ~= nil then
      local _,_,agat,agath                                         = string.find(getglobal("PlayerStatFrameLeft2").tooltip2,"^.* by (%d+\.%d+).*by (%d+)");
      if agat == nil then
         _,_,agat,agath                                            = string.find(getglobal("PlayerStatFrameLeft2").tooltip2,"^.* um (%d+\.%d+).*um (%d+)");
      end
      UploadData.agility_critHitPercent                            = agat;
      UploadData.agility_armor                                     = agath;
   end
   if getglobal("PlayerStatFrameLeft3").tooltip2 ~= nil then
      local _,_,stmat                                              = string.find(getglobal("PlayerStatFrameLeft3").tooltip2,"^.*Health by (%d+)");
      if stmat == nil then
         _,_,stmat                                                =  string.find(getglobal("PlayerStatFrameLeft3").tooltip2,"^.* um (%d+)");
      end
      UploadData.stamina_health                                    = stmat;
   end
   if getglobal("PlayerStatFrameLeft4").tooltip2 ~= nil then
      local _,_,inat,inath                                        = string.find(getglobal("PlayerStatFrameLeft4").tooltip2,"^.* by (%d+).*by (%d+\.%d+)");
      if inat == nil then
         _,_,inat,inath                                     = string.find(getglobal("PlayerStatFrameLeft4").tooltip2,"^.* um (%d+).*um (%d+\.%d+)");
      end
      UploadData.intellect_mana                                = inat;
      UploadData.intellect_critHitPercent                      = inath;
   end
   if getglobal("PlayerStatFrameLeft5") ~= nil then
      local _,_,spiat,spiath                                   = string.find(getglobal("PlayerStatFrameLeft5").tooltip2,"^.* by (%d+).*by (%d+)");
      if spiat == nil then
         _,_,spiat,spiath                                      = string.find(getglobal("PlayerStatFrameLeft5").tooltip2,"^.* um (%d+).*um (%d+)");
      end
         UploadData.spirit_healthRegen                                = spiat;
         UploadData.spirit_manaRegen                                  = spiath;
   end
   if  getglobal("PlayerStatFrameLeft6").tooltip2 ~= nil then
       local _,_,armat                                              = string.find(getglobal("PlayerStatFrameLeft6").tooltip2,"^.*by (%d+\.%d+)");
       if armat == nil then
          _,_,armat                                              = string.find(getglobal("PlayerStatFrameLeft6").tooltip2,"^.*um (%d+\.%d+)");
       end
       UploadData.armor_reductionPercent                            = armat;
   end

end

function xChar:GetPVP()
	UploadData.pvp_rank = UnitPVPRank("player")
	UploadData.pvp_hk, UploadData.pvp_highestrank = GetPVPLifetimeStats()
	UploadData.pvp_honor = GetHonorCurrency()
	UploadData.pvp_yhk, UploadData.pvp_ycp = GetPVPYesterdayStats()
end

function xChar:GetProfileData()
	UploadData.Level = UnitLevel("player")
	UploadData.Realm = GetRealmName()
	UploadData.Gender = UnitSex("player")
	UploadData.Race = UnitRace("player")
	UploadData.Class = UnitClass("player")
	UploadData.Faction = UnitFactionGroup("player")
	UploadData.Name = UnitName("player")

	UploadData.Title = GetTitleName(GetCurrentTitle())
	UploadData.Armor = UnitArmor("player")
	
	_, UploadData.resistances_fire = UnitResistance("player",2)
	_, UploadData.resistances_nature = UnitResistance("player",3)
	_, UploadData.resistances_frost = UnitResistance("player",4)
	_, UploadData.resistances_shadow = UnitResistance("player",5)
	_, UploadData.resistances_arcane = UnitResistance("player",6)

	UploadData.mana	= UnitMana("player")
	UploadData.max_mana	= UnitManaMax("player")
	UploadData.health = UnitHealth("player")
	UploadData.max_health = UnitHealthMax("player")
	UploadData.Money = GetMoney()
	UploadData.XP = UnitXP("player")
	UploadData.XPMax = UnitXPMax("player")

end

function xChar:GetTalentShort()
	local talentName, talentIcon, talentPoints, _ = GetTalentTabInfo(1)
	UploadData.talent1_type  = talentName
	UploadData.talent1_value = talentPoints
	UploadData.talent1_icon  = talentIcon
	talentName, talentIcon, talentPoints, _ = GetTalentTabInfo(2)
	UploadData.talent2_type  = talentName
	UploadData.talent2_value = talentPoints
	UploadData.talent2_icon  = talentIcon
	talentName, talentIcon, talentPoints, _ = GetTalentTabInfo(3)
	UploadData.talent3_type  = talentName
	UploadData.talent3_value = talentPoints
	UploadData.talent3_icon  = talentIcon
end

function xChar:GetAddons()
	UploadData.AddOns = {}
	for i = 1, GetNumAddOns() do
		local name, title, notes, enabled, loadable, reason, security = GetAddOnInfo(i)
		UploadData.AddOns[i] = {}
		UploadData.AddOns[i].name = name
		UploadData.AddOns[i].title = title
		UploadData.AddOns[i].notes = ""
		UploadData.AddOns[i].enabled = enabled
		UploadData.AddOns[i].reason	= reason
		UploadData.AddOns[i].security = security
		UploadData.AddOns[i].url = GetAddOnMetadata(i, "URL")
	end
end

function xChar:GetSkill()
	if GetNumSkillLines() == 0 then return end
	local ParentId
	local Prof = 0
	UploadData.skill = {}
	for i = 1, GetNumSkillLines() do
		local skillName, isHeader, isExpanded, skillRank, numTempPoints, skillModifier, skillMaxRank,
		isAbandonable, stepCost, rankCost, minLevel, skillCostType, skillDescription = GetSkillLineInfo(i)
		
		UploadData.skill[i]				= {}
		UploadData.skill[i].skill_data_id	= i
-- 		why would i use up space for already known data?	
		UploadData.skill[i].Realm			= GetRealmName()
		UploadData.skill[i].Name			= UnitName("player")

		if isHeader then
			UploadData.skill[i].skill_data_parent_id = 0
			ParentId = i
			if skillName == L["Professions"] then
				Prof = 1
			else
				Prof = 0
			end
		else
			UploadData.skill[i].skill_data_parent_id = ParentId
			if Prof ~= 0 then
				UploadData["primaryProfession".. Prof .."_value"] = skillRank .. " / " .. skillMaxRank
				UploadData["primaryProfession".. Prof .."_type"] = skillName
				Prof = Prof + 1
			end
		end
		UploadData.skill[i].skill_data_name = skillName
		if skillMaxRank ~= 0 then
			UploadData.skill[i].skill_data_rank = skillRank
			UploadData.skill[i].skill_data_max_rank = skillMaxRank
		end
	end
end

function xChar:GetReputation()
	if GetNumFactions() == 0 then return end
	local ParentId
	UploadData.reputation = {}

	for i = 1, GetNumFactions() do
		local name, description, standingId, bottomValue, topValue, earnedValue, atWarWith, 
		canToggleAtWar, isHeader, _, _ = GetFactionInfo(i)
		
		UploadData.reputation[i]						= {}
		UploadData.reputation[i].reputation_data_id	= i
		-- why use up space for already known data?
		UploadData.reputation[i].Name					= UnitName("player")
		UploadData.reputation[i].Realm					= GetRealmName()
		
		if isHeader then
			UploadData.reputation[i].reputation_data_parent_id = 0
			ParentId = i
		else
			UploadData.reputation[i].reputation_data_parent_id = ParentId
		end
		
		UploadData.reputation[i].reputation_data_name = name
		
		if standingId == 0 then
			UploadData.reputation[i].reputation_data_status = "Unknown"
		elseif standingId == 1 then
			UploadData.reputation[i].reputation_data_status = "Hated"
		elseif standingId == 2 then
			UploadData.reputation[i].reputation_data_status = "Hostile"
		elseif standingId == 3 then
			UploadData.reputation[i].reputation_data_status = "Unfriendly"
		elseif standingId == 4 then
			UploadData.reputation[i].reputation_data_status = "Neutral"
		elseif standingId == 5 then
			UploadData.reputation[i].reputation_data_status = "Friendly"
		elseif standingId == 6 then
			UploadData.reputation[i].reputation_data_status = "Honored"
		elseif standingId == 7 then
			UploadData.reputation[i].reputation_data_status = "Revered"
		elseif standingId == 8 then
			UploadData.reputation[i].reputation_data_status = "Exalted"
		end
		
		if earnedValue < 0 then
			UploadData.reputation[i].reputation_data_max_rank	= bottomValue - topValue
			UploadData.reputation[i].reputation_data_rank		= bottomValue - earnedValue
		else
			UploadData.reputation[i].reputation_data_max_rank	= topValue - bottomValue
			UploadData.reputation[i].reputation_data_rank		= earnedValue - bottomValue
		end
	end
end


function xChar:GetInventoryItemAce(Slot)
	local item = {}
	item.ItemLink = GetInventoryItemLink("player", GetInventorySlotInfo(Slot))
	if item.ItemLink then
	
		_, _, item.itemid, item.enchant, item.socket1, item.socket2, item.socket3, item.temp, item.subid = 
		string.find(item.ItemLink, "item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):")
		
		item.Name = GetItemInfo(item.ItemLink)
		item.isBroken = GetInventoryItemBroken("player", Slot)
		
	end
	return item
end

function xChar:GetInventoryItem(Slot)
	 local ItemLink 				   = GetInventoryItemLink("player",GetInventorySlotInfo(Slot));
	 if ItemLink ~= nil then
	  local _,_,ItemId                                      = string.find(ItemLink,"^.*Hitem:(%d+)");
          local _,_,_,_,_,_,_,_,_,itemName = GetItemInfo(ItemLink)
          local Tooltip                                    = self:GetSlotTooltipInfo(Slot)
          return itemName,ItemId,Tooltip;
	 end
end

function xChar:GetContainerItem(bag, slot)
	local item = {}
	item.link = GetContainerItemLink(bag, slot)
	if item.link then
	
		_, _, item.itemid, item.enchant, item.socket1, item.socket2, item.socket3, item.temp, item.subid = 
		string.find(item.link, "item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):")
		
		item.name = GetItemInfo(item.link)
		_, item.count = GetContainerItemInfo(bag, slot)		
		
	end
	return item
end

function xChar:GetInventoryItemsAce()
	Slots = {"HeadSlot", "NeckSlot", "ShoulderSlot", "BackSlot", "ChestSlot", "ShirtSlot", "TabardSlot",
	"WristSlot", "HandsSlot", "WaistSlot", "LegsSlot", "FeetSlot", "Finger0Slot", "Finger1Slot", "Trinket0Slot",
	"Trinket1Slot", "MainHandSlot", "SecondaryHandSlot", "RangedSlot", "AmmoSlot"}
	
	UploadData.Inventory = {}
	local Inventory = {}
	
	for k, invSlot in ipairs(Slots) do
		Inventory[invSlot] = self:GetInventoryItem(invSlot)
	end
	
	UploadData.Inventory = Inventory
end
function xChar:GetInventoryItems()
         Slots = {"HeadSlot","NeckSlot","ShoulderSlot","BackSlot","ChestSlot","ShirtSlot","TabardSlot","WristSlot","HandsSlot"
         ,"WaistSlot","LegsSlot" ,"FeetSlot","Finger0Slot","Finger1Slot","Trinket0Slot","Trinket1Slot","MainHandSlot"
         ,"SecondaryHandSlot","RangedSlot","AmmoSlot"}
         UploadData.item_head,UploadData.item_head_id,UploadData.item_head_tooltip	                      = self:GetInventoryItem("HeadSlot");
         UploadData.item_neck,UploadData.item_neck_id,UploadData.item_neck_tooltip	                      = self:GetInventoryItem("NeckSlot");
         UploadData.item_shoulders,UploadData.item_shoulder_id,UploadData.item_shoulder_tooltip	              = self:GetInventoryItem("ShoulderSlot");
         UploadData.item_back,UploadData.item_back_id,UploadData.item_back_tooltip	                      = self:GetInventoryItem("BackSlot");
         UploadData.item_chest,UploadData.item_chest_id,UploadData.item_chest_tooltip	                      = self:GetInventoryItem("ChestSlot");
         UploadData.item_shirt,UploadData.item_shirt_id,UploadData.item_shirt_tooltip	                      = self:GetInventoryItem("ShirtSlot");
         UploadData.item_tabard,UploadData.item_tabard_id,UploadData.item_tabard_tooltip	              = self:GetInventoryItem("TabardSlot");
         UploadData.item_wrist,UploadData.item_wrist_id,UploadData.item_wrist_tooltip	                      = self:GetInventoryItem("WristSlot");
         UploadData.item_hands,UploadData.item_hands_id,UploadData.item_hands_tooltip	                      = self:GetInventoryItem("HandsSlot");
         UploadData.item_waist,UploadData.item_waist_id,UploadData.item_waist_tooltip	                      = self:GetInventoryItem("WaistSlot");
         UploadData.item_legs,UploadData.item_legs_id,UploadData.item_legs_tooltip	                      = self:GetInventoryItem("LegsSlot");
         UploadData.item_feet,UploadData.item_feet_id,UploadData.item_feet_tooltip	                      = self:GetInventoryItem("FeetSlot");
         UploadData.item_finger1,UploadData.item_finger1_id,UploadData.item_finger1_tooltip	              = self:GetInventoryItem("Finger0Slot");
         UploadData.item_finger2,UploadData.item_finger2_id,UploadData.item_finger2_tooltip	              = self:GetInventoryItem("Finger1Slot");
         UploadData.item_trinket1,UploadData.item_trinket1_id,UploadData.item_trinket1_tooltip	              = self:GetInventoryItem("Trinket0Slot");
         UploadData.item_trinket2,UploadData.item_trinket2_id,UploadData.item_trinket2_tooltip	              = self:GetInventoryItem("Trinket1Slot");
         UploadData.item_mainHand,UploadData.item_mainHand_id,UploadData.item_mainHand_tooltip	              = self:GetInventoryItem("MainHandSlot");
         UploadData.item_offHand,UploadData.item_offHand_id,UploadData.item_offHand_tooltip	              = self:GetInventoryItem("SecondaryHandSlot");
         UploadData.item_ranged,UploadData.item_ranged_id,UploadData.item_ranged_tooltip	              = self:GetInventoryItem("RangedSlot");
         UploadData.item_ammo,UploadData.item_ammo_id,UploadData.item_ammo_tooltip	                      = self:GetInventoryItem("AmmoSlot");
end

function xChar:GetBagsAce()

	UploadData.bags = {}
	for i = 0, 4 do 
		local BagName = GetBagName(i)
		if BagName then
			local bag = {}
			bag.BagName = BagName
			bag.BagSize = GetContainerNumSlots(i)
			bag.BagId = i
			bag.Slot = {}
			for j = 1, bag.BagSize do
				bag.Slot[j] = self:GetContainerItem(i, j)
			end
		end
		UploadData.bags[i] = bag
	end
end		
function xChar:GetBags()
         UploadData.bags                                           = {}
         local id                                                 = 1;
         for i = 0,4 do
             local BagName                                         = GetBagName(i);
             if BagName ~= nil then
                UploadData.bags[id]                                = {}
                UploadData.bags[id].BagName			   = BagName;
                UploadData.bags[id].BagSize                        = GetContainerNumSlots(i);
                UploadData.bags[id].BagId                          = i;
                id                                                 = id + 1;
                for j = 1, GetContainerNumSlots(i) do
                    local ItemLink 				   = GetContainerItemLink(i,j);
                    if ItemLink ~= nil then
                       local itemName,_,itemRarity,itemLevel,itemMinLevel,itemType,itemSubType,itemStackCount,itemEquipLoc,itemTexture = GetItemInfo(ItemLink)
                       local _,_,ItemId                             = string.find(ItemLink,"^.*Hitem:(%d+)");
                       UploadData.bags[id]                          = {}
                       UploadData.bags[id].BagItemName              = itemTexture;
                       UploadData.bags[id].BagItemId                = ItemId;
             	       UploadData.bags[id].BagItemBagId             = i;
             	       UploadData.bags[id].BagItemAmount            = itemStackCount;
             	       UploadData.bags[id].BagItemTooltip           = self:GetBagTooltipInfo(i,j);
                       id                                           = id + 1;
                     end
                end
             end
         end
         
end

function xChar:PLAYER_REGEN_DISABLED()

end

function xChar:isBoss(msg)

	for en, localized in pairs(self.BossTable) do
		-- self:Print(en, localized)
		if strfind(msg, localized) then 
			return en 
		end
	end
	-- self:Print(msg .. " isn't a Boss.")
	return false
	
end


function xChar:CHAT_MSG_COMBAT_HOSTILE_DEATH()
	
	local name = self:isBoss(arg1)
	
	if name then 
		if not UploadData.Bosskills then UploadData.Bosskills = {} end
		if not UploadData.BossPoints then UploadData.BossPoints = {} end
		
		self:Print(name .. L[" was killed."])
		UploadData.Bosskills[name] = 1
		
		if BZ:HasReverseTranslation(GetRealZoneText()) then 
			local zone = BZ:GetReverseTranslation(GetRealZoneText())
			local points = UploadData.BossPoints[zone] or 0
			UploadData.BossPoints[zone] = self.BossPoints[name] + points
			self:Print(zone, UploadData.BossPoints[zone])
		else
			local zone = GetRealZoneText()
			local points = UploadData.BossPoints[zone] or 0
			UploadData.BossPoints[zone] = self.BossPoints[name] + points
			self:Print(zone, UploadData.BossPoints[zone])
		end
		
		if UploadData.BossPoints[zone] > 100 then 
			UploadData.BossPoints[zone] = 100 
		end
	end

end

function xChar:PLAYER_REGEN_ENABLED()

end

function xChar:createBossTable()

	self.BossTable = {}
	-- for testing on non-bosses
	-- self.BossTable["Skettis Surger"] = "Skettis Surger"

	self.BossTable["Attumen the Huntsman"] = BB["Attumen the Huntsman"]
	self.BossTable["Dorothee"] = BB["Dorothee"]
	self.BossTable["Hyakiss the Lurker"] = BB["Hyakiss the Lurker"]
	self.BossTable["Julianne"] = BB["Julianne"]
	self.BossTable["Maiden of Virtue"] = BB["Maiden of Virtue"]
	self.BossTable["Midnight"] = BB["Midnight"]
	self.BossTable["Moroes"] = BB["Moroes"]
	self.BossTable["Netherspite"] = BB["Netherspite"]
	self.BossTable["Nightbane"] = BB["Nightbane"]
	self.BossTable["Prince Malchezaar"] = BB["Prince Malchezaar"]
	self.BossTable["Roar"] = BB["Roar"]
	self.BossTable["Rokad the Ravager"] = BB["Rokad the Ravager"]
	self.BossTable["Romulo"] = BB["Romulo"]
	self.BossTable["Shade of Aran"] = BB["Shade of Aran"]
	self.BossTable["Shadikith the Glider"] = BB["Shadikith the Glider"]
	self.BossTable["Strawman"] = BB["Strawman"]
	self.BossTable["Terestian Illhoof"] = BB["Terestian Illhoof"]
	self.BossTable["The Big Bad Wolf"] = BB["The Big Bad Wolf"]
	self.BossTable["The Crone"] = BB["The Crone"]
	self.BossTable["The Curator"] = BB["The Curator"]
	self.BossTable["Tinhead"] = BB["Tinhead"]
	self.BossTable["Tito"] = BB["Tito"]
	
	self.BossTable["Fathom-Lord Karathress"] = BB["Fathom-Lord Karathress"]
	self.BossTable["Hydross the Unstable"] = BB["Hydross the Unstable"]
	self.BossTable["Lady Vashj"] = BB["Lady Vashj"]
	self.BossTable["Leotheras the Blind"] = BB["Leotheras the Blind"]
	self.BossTable["Morogrim Tidewalker"] = BB["Morogrim Tidewalker"]
	self.BossTable["The Lurker Below"] = BB["The Lurker Below"]
	
	self.BossTable["Gruul the Dragonkiller"] = BB["Gruul the Dragonkiller"]
	self.BossTable["High King Maulgar"] = BB["High King Maulgar"]

	self.BossTable["Magtheridon"] = BB["Magtheridon"]

	self.BossTable["Al'ar"] = BB["Al'ar"]
	self.BossTable["High Astromancer Solarian"] = BB["High Astromancer Solarian"]
	self.BossTable["Kael'thas Sunstrider"] = BB["Kael'thas Sunstrider"]
	self.BossTable["Void Reaver"] = BB["Void Reaver"]
	
	self.BossTable["Illidan Stormrage"] = BB["Illidan Stormrage"]
	self.BossTable["Mother Shahraz"] = BB["Mother Shahraz"]
	self.BossTable["Lady Malande"] = BB["Lady Malande"]
	self.BossTable["High Warlord Naj'entus"] = BB["High Warlord Naj'entus"]
	self.BossTable["Teron Gorefiend"] = BB["Teron Gorefiend"] 
	self.BossTable["Gathios the Shatterer"] = BB["Gathios the Shatterer"]
	self.BossTable["Veras Darkshadow"] = BB["Veras Darkshadow"]
	self.BossTable["Gurtogg Bloodboil"] = BB["Gurtogg Bloodboil"] 
	self.BossTable["High Nethermancer Zerevor"] = BB["High Nethermancer Zerevor"]
	self.BossTable["Essence of Desire"] = BB["Essence of Desire"]
	self.BossTable["Essence of Grief"] = BB["Essence of Grief"]
	self.BossTable["Essence of Anger"] = BB["Essence of Anger"]
	self.BossTable["Supremus"] = BB["Supremus"]


	self.BossPoints = {}
	-- for testing on non-bosses
	-- self.BossPoints["Skettis Surger"] = "Skettis Surger"

	self.BossPoints["Attumen the Huntsman"] = 4
	self.BossPoints["Midnight"] = 4

	self.BossPoints["Dorothee"] = 1
	self.BossPoints["Roar"] = 1
	self.BossPoints["Tinhead"] = 1
	self.BossPoints["Tito"] = 1
	self.BossPoints["Strawman"] = 1
	self.BossPoints["The Crone"] = 1

	self.BossPoints["Julianne"] = 4
	self.BossPoints["Romulo"] = 4

	self.BossPoints["The Big Bad Wolf"] = 8

	self.BossPoints["Maiden of Virtue"] = 8
	self.BossPoints["Moroes"] = 8
	self.BossPoints["Netherspite"] = 8
	self.BossPoints["Nightbane"] = 11
	self.BossPoints["Prince Malchezaar"] = 8
	self.BossPoints["Shade of Aran"] = 8
	self.BossPoints["Terestian Illhoof"] = 8
	self.BossPoints["The Curator"] = 8

	self.BossPoints["Hyakiss the Lurker"] = 1
	self.BossPoints["Shadikith the Glider"] = 1
	self.BossPoints["Rokad the Ravager"] = 1
	
	self.BossPoints["Fathom-Lord Karathress"] = 16
	self.BossPoints["Hydross the Unstable"] = 16
	self.BossPoints["Lady Vashj"] = 20
	self.BossPoints["Leotheras the Blind"] = 16
	self.BossPoints["Morogrim Tidewalker"] = 16
	self.BossPoints["The Lurker Below"] = 16
	
	self.BossPoints["Gruul the Dragonkiller"] = 50
	self.BossPoints["High King Maulgar"] = 50

	self.BossPoints["Magtheridon"] = 100

	self.BossPoints["Al'ar"] = 25
	self.BossPoints["High Astromancer Solarian"] = 25
	self.BossPoints["Kael'thas Sunstrider"] = 25
	self.BossPoints["Void Reaver"] = 25

	self.BossPoints["Illidan Stormrage"] = 9
	self.BossPoints["Mother Shahraz"] = 7
	self.BossPoints["Lady Malande"] = 7
	self.BossPoints["High Warlord Naj'entus"] = 7
	self.BossPoints["Teron Gorefiend"] = 7
	self.BossPoints["Gathios the Shatterer"] = 7
	self.BossPoints["Veras Darkshadow"] = 7
	self.BossPoints["Gurtogg Bloodboil"] = 7
	self.BossPoints["High Nethermancer Zerevor"] = 7
	self.BossPoints["Essence of Desire"] = 7
	self.BossPoints["Essence of Grief"] = 7
	self.BossPoints["Essence of Anger"] = 7
	self.BossPoints["Supremus"] = 7

end

function xChar:GetTalents()
	UploadData.talent = {}
	UploadData.talent.talent_data_query = self:GetTalentQuery()
	local _, PlayerClass = UnitClass("player")
	UploadData.talent.talent_data_class = PlayerClass
end
